/*
**> American Option Example
**
*/

new;
library pde,pgraph;
PDEset;

#include option.src;

declare matrix exercice;

cls;

K = 100;
sigma = 0.2;
tau = 0.25;
r = 0.05;
b = 0;

xmin = 50; xmax = 200; N = 251;
tmin =  0; tmax = tau; M = 501;


proc aProc(t,x);
  retp( 0.5*(sigma^2) .* (x^2) );
endp;

proc bProc(t,x);
  retp( b .* x );
endp;

proc cProc(t,x);
  retp( r .* ones(rows(x),1) );
endp;

proc dProc(t,x);
  retp( zeros(rows(x),1) );
endp;


/*
**> Prise en compte d'une ingalit variationnelle
*/

proc eProc(t,x,U);
  local PayOff,cnd;
  
  PayOff = x - K;
  PayOff = PayOff .* (PayOff .> 0);
  cnd = PayOff .> U;
  exercice = exercice ~ cnd;
  retp( cnd .* PayOff + (1-cnd) .* U );
endp;

proc tminBound(t,x);
  local PayOff;
  PayOff = x - K;
  PayOff = PayOff .* (PayOff .> 0);
  retp( PayOff );
endp;

proc xminBound(t,x);
  retp( zeros(rows(x),1) );
endp;

proc xmaxBound(t,x);
  local PayOff;
  PayOff = x - K;
  PayOff = PayOff .* (PayOff .> 0);
  retp( PayOff );
endp;

proc DxmaxBound(t,x);
  retp( ones(rows(x),1) );
endp;

_PDE_SaveLastIter = 1;
_PDE_PrintIters = 50;


/* Neumann "user" condition */

call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
         &tminBound,&xminBound,0,0,&DxmaxBound);


t = GenerateGrid1(tmin,tmax,M);
x = GenerateGrid1(xmin,xmax,N);
exercice = {};
call solvePDE(t,x,"uFile",0.5);
S1 = readPDE("uFile","x");
U1 = readPDE("uFile","t"|tau);
exercice1 = S1[maxindc(exercice)];

proc invcdf(u);
  local delta,x;

  u = substute(u,u .== 0,1e-7);
  u = substute(u,u .== 1,1-1e-3);

  delta = exp((b-r)*tau)*u;
  x = invEuropeanCallDelta(delta,K,sigma,tau,b,r);
 
  retp(x);
endp;

x = GenerateGrid2(xmin,xmax,N,&invcdf);
exercice = {};
call solvePDE(t,x,"uFile",0.5);
S2 = readPDE("uFile","x");
U2 = readPDE("uFile","t"|tau);
exercice2 = S2[maxindc(exercice)];

xstar = K;
x = GenerateGrid3(xmin,xmax,N,xstar,2);
exercice = {};
call solvePDE(t,x,"uFile",0.5);
S3 = readPDE("uFile","x");
U3 = readPDE("uFile","t"|tau);
exercice3 = S3[maxindc(exercice)];

xstar = 112.5;
x = GenerateGrid3(xmin,xmax,N,xstar,2);
exercice = {};
call solvePDE(t,x,"uFile",0.5);
S4 = readPDE("uFile","x");
U4 = readPDE("uFile","t"|tau);
exercice4 = S4[maxindc(exercice)];
4

graphset;
  begwind;
  window(2,2,0);

  _pnum = 2; _pltype = 6; 
  _pnumht = 0.17; _paxht = 0.25; _ptitlht = 0.25; 
  _pframe = 0;

  t_ = miss(t,0);

  ytics(100,125,5,2);
  xtics(0,0.25,0.05,2);

  xlabel("t");
  ylabel("S[*]");
  
  setwind(1);
    title("Grid 1");
    xy(t_,exercice1);

  setwind(2);
    title("Grid 2");
    xy(t_,exercice2);

  setwind(3);
    title("Grid 3");
    xy(t_,exercice3);

  setwind(4);
    title("Grid 4");
    xy(t_,exercice4);

    graphprt("-c=1 -cf=am.ps");
  endwind;