/*
**> Binary Option Example
**
*/

new;
library pde,pgraph;
PDEset;

#include option.src;

cls;

K = 100;
sigma = 0.2;
tau = 0.25;
r = 0.05;
b = 0;

xmin = 50; xmax = 150; N = 101;
tmin =  0; tmax = tau; M = 251;

proc aProc(t,x);
  retp( 0.5*(sigma^2) .* (x^2) );
endp;

proc bProc(t,x);
  retp( b .* x );
endp;

proc cProc(t,x);
  retp( r .* ones(rows(x),1) );
endp;

proc dProc(t,x);
  retp( zeros(rows(x),1) );
endp;

proc tminBound(t,x);
  retp( (x - K).> 0 );
endp;

proc xminBound(t,x);
  retp( zeros(rows(x),1) );
endp;

proc xmaxBound(t,x);
  retp( exp(-r*t) );
endp;

proc DxminBound(t,x);
  retp( zeros(rows(x),1) );
endp;

proc DxmaxBound(t,x);
  retp( zeros(rows(x),1) );
endp;

_PDE_SaveLastIter = 1;
_PDE_PrintIters = 50;


/* Neumann "user" condition */

call PDE(&aProc,&bProc,&cProc,&dProc,0,
         &tminBound,&xminBound,0,0,&DxmaxBound);


t = GenerateGrid1(tmin,tmax,M);
xstar = K;

x = GenerateGrid3(xmin,xmax,N,xstar,200);
call solvePDE(t,x,"uFile",0.5);
S1 = readPDE("uFile","x");
U1 = readPDE("uFile","t"|tau);

x = GenerateGrid3(xmin,xmax,N,xstar,20);
call solvePDE(t,x,"uFile",0.5);
S2 = readPDE("uFile","x");
U2 = readPDE("uFile","t"|tau);

x = GenerateGrid3(xmin,xmax,N,xstar,2);
call solvePDE(t,x,"uFile",0.5);
S3 = readPDE("uFile","x");
U3 = readPDE("uFile","t"|tau);

U = U1~U2~U3;
S = S1~S2~S3;

U0 = BinaryCall(S,K,sigma,tau,b,r);

Delta = BinaryCallDelta(S,K,sigma,tau,b,r);
Gamma_ = BinaryCallGamma(S,K,sigma,tau,b,r);
{d1,d2} = derivePDE(s,u);


sigma_ = sigma;
sigma = 1.0001*sigma;

x = generateGrid3(xmin,xmax,N,xstar,200);
call solvePDE(t,x,"uFile",0.5);
S1 = readPDE("uFile","x");
U1 = readPDE("uFile","t"|tau);

x = generateGrid3(xmin,xmax,N,xstar,20);
call solvePDE(t,x,"uFile",0.5);
S2 = readPDE("uFile","x");
U2 = readPDE("uFile","t"|tau);

x = generateGrid3(xmin,xmax,N,xstar,2);
call solvePDE(t,x,"uFile",0.5);
S3 = readPDE("uFile","x");
U3 = readPDE("uFile","t"|tau);

d3 = ((U1~U2~U3)-U)./(sigma-sigma_);
Vega = -BinaryCallVega(S,K,sigma,tau,b,r);


graphset;
  begwind;
  window(2,2,0);

  _pdate = "";
  fonts("simplex simgrma");
  _pcross = 0; _pframe = 0; _pnum = 2; _pysci = 0;
  _pnumht = 0.20; _paxht = 0.20; _ptitlht = 0.20;

  _pltype = 6|1|3;
  xtics(70,140,10,2);
  ytics(-0.005,0.020,0.005,2);
  _plegstr = "\202a\201 = 200\000\202a\201 = 20\000\202a\201 = 2";
  xlabel("\212S]0[");
  ylabel("error");

  setwind(1);
    title("Call");
    ytics(-0.005,0.020,0.005,2);
    xy(S,U0-U);

  setwind(2);
    title("Delta");
    ytics(-0.0015,0.0015,0.0005,2);
    xy(S,delta-d1);

  setwind(3);
    title("Gamma");
    ytics(-0.00025,0.00015,0.00005,2);
    xy(S,gamma_-d2);

  setwind(4);
   _pysci = 1;
    title("Vega");
    ytics(-0.10,0.05,0.05,2);
    _plegctl = {2 6 5.75 1.75};
    xy(S,vega-d3);
    graphprt("-c=1 -cf=binary.ps");
  endwind;