/*
**> European Option Example
**
** When non-uniform grids do not work ?
**
*/

new;
library PDE,pgraph;
PDEset;

#include option.src;

cls;

K = 100;
sigma = 0.2;
tau = 0.25;
r = 0.05;
b = 0;

xmin = 50; xmax = 150; N = 151;
tmin =  0; tmax = tau; M = 251;

proc aProc(t,x);
  retp( 0.5*(sigma^2) .* (x^2) );
endp;

proc bProc(t,x);
  retp( b .* x );
endp;

proc cProc(t,x);
  retp( r .* ones(rows(x),1) );
endp;

proc dProc(t,x);
  retp( zeros(rows(x),1) );
endp;

proc tminBound(t,x);
  local PayOff;
  PayOff = x - K;
  PayOff = PayOff .* (PayOff .> 0);
  retp( PayOff );
endp;

proc xminBound(t,x);
  retp( zeros(rows(x),1) );
endp;

proc xmaxBound(t,x);
  local PayOff;
  PayOff = x - K;
  PayOff = PayOff .* (PayOff .> 0);
  retp( PayOff );
endp;

proc DxmaxBound(t,x);
  retp( ones(rows(x),1) );
endp;

_PDE_SaveLastIter = 1;
_PDE_PrintIters = 50;


/* Neumann "user" condition */

call PDE(&aProc,&bProc,&cProc,&dProc,0,
         &tminBound,&xminBound,0,0,&DxmaxBound);


t = GenerateGrid1(tmin,tmax,M);

let N = 151 1001 151 1001;

x = GenerateGrid3(xmin,xmax,N[1],K,2);
call solvePDE(t,x,"uFile",0.5);
x1 = readPDE("uFile","x");
U1 = readPDE("uFile","t"|tau);

x = GenerateGrid3(xmin,xmax,N[2],K,2);
call solvePDE(t,x,"uFile",0.5);
x2 = readPDE("uFile","x");
U2 = readPDE("uFile","t"|tau);

x = GenerateGrid1(xmin,xmax,N[3]);
call solvePDE(t,x,"uFile",0.5);
x3 = readPDE("uFile","x");
U3 = readPDE("uFile","t"|tau);

x = GenerateGrid1(xmin,xmax,N[4]);
call solvePDE(t,x,"uFile",0.5);
x4 = readPDE("uFile","x");
U4 = readPDE("uFile","t"|tau);


S = miss(zeros(maxc(N),4),0);
U = miss(zeros(maxc(N),4),0);

i = 1;
do until i > 4;
  S[1:N[i],i] = varget(ftos(i,"x%lf",1,0));
  U[1:N[i],i] = varget(ftos(i,"U%lf",1,0));
  i = i + 1;
endo;

C0 = EuropeanCall(S,K,sigma,tau,b,r);

let string title_[4] = "Grid 1 & N = 151 " 
                       "Grid 1 & N = 1001 "
                       "Grid 3 & N = 151 "
                       "Grid 3 & N = 1001 ";
                        
graphset;
  begwind;
  window(2,2,0);

  _pdate = "";
  _pcross = 0; _pframe = 0; _pnum = 2;
  _pltype = 6|1|3;
  xtics(80,120,10,2);
  ytics(-0.001,0.0025,0.0005,2);
  xlabel("\212S]0[");
  ylabel("\212Error");
  _pysci = 0;
  _pnumht = 0.20; _paxht = 0.20; _ptitlht = 0.20;

  i = 1;
  do until i > 4;
    setwind(i);
    title(title_[i]);
    xy(S[.,i],C0[.,i]-U[.,i]);
    i = i + 1;
  endo;

  graphprt("-c=1 -cf=wrong1.ps");

  endwind;